/*******************************************************************************

    HEASARC Database SQLALL (HDBSQLALL) server utility.

    AUTHOR: Song Yom (HSTX)
    DATE:   11/94

    MODIFICATION HISTORY:
    DATE    PROGRAMMER      DESCRIPTION
    ----    ----------      -----------
    12/94   Song Yom        added passing of sign argument to dec_deg2dms
                            autil routine
    06/95   Song Yom        removed host dependent port number definition

*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "csios.h"
#include "autil.h"
#include "util.h"

/*
    TCP/IP communication socket port number
*/

#define PORT 57351

/*
    Maximum length of data buffer and number of tokens (field and value)
*/

#define MAXLEN CSIOS_SIZE
#define MAXTOK 1024

/*
    SQL command and buffer definitions
*/

#define SQLCOM "hdbsql \"output ads;dump format on;equinox %d;select * from %s %s\""
#define SEPARATOR "|"
#define LINE "-------------------------------------------------------------------------------"

/*
    Database parameter file directory definition
*/

#define PRFILES "/dbase/pr_files/"

#ifdef sun
extern void hdbi2class();
extern void hdbshf2atim();
extern void slaPreces();
#else
extern void hdbi2class(int,char *);
extern void hdbshf2atim(int,char *);
extern void slaPreces(char [3],double,double,double *,double *);
#endif

main()
{
    int iostatus;
    int length;
    int i;
    int j;
    int k;
    int n;
    int ntfield;
    int count;
    int found;
    int iequinox;
    int nra = -1;
    int ndec = -1;
    int npra = -1;
    int npdec = -1;
    int nclass = -1;
    int iclass;
    int shf;
    int hh;
    int dd;
    int mm;
    float ss;
    double ra;
    double dec;
    double pra;
    double pdec;
    char sign;
    char *pb;
    char *field[MAXTOK];
    char *value[MAXTOK];
    char *tfield[MAXTOK];
    char fields[MAXLEN];
    char tfields[MAXLEN];
    char equinox[MAXLEN];
    char table[MAXLEN];
    char search[MAXLEN];
    char file[MAXLEN];
    char buffer[MAXLEN];
    char ibuffer[MAXLEN];
    char obuffer[MAXLEN];
    FILE *fp;
    FILE *pp;

/*
    Establish the server environment, and read, parse, and verify the
    parameter buffer from the client.  If successful, execute HEASARC
    hdbsql client software via the pipe.  Determine the status of the query,
    and if successful, construct and send data buffer(s) to the client.  Send
    error message if any to the client.

    PROCESSING PROCEDURE:

    1.  parse the field data into tokens
    2.  read the table parameter definition file to save the SHF format time
        field(s)
    3.  save the token numbers of coordinate fields: RA, DEC, POINTRA,
        POINTDEC, POINT_RA, and POINT_DEC
    4.  save the token number for CLASS field
    5.  parse the value data into tokens
    6.  do ASCII to double conversion of coordinate values, and perform
        coodinate precession
    7.  do integer to ASCII conversion of CLASS field data
    8.  do SHF to ASCII conversion of SHF format field data
    9.  construct and print field=value pair formatted data to standard output
    10. repeat steps 3 through 5 until data is exhausted
*/

    if (csios_init(PORT) == CSIOS_SUCCESS)
    {
        if ((iostatus = csios_input(buffer,&length)) == CSIOS_SUCCESS)
        {
            strtrm(buffer);
            fprintf(stderr,"server> %s\n",buffer);
            strcpy(equinox,buffer);
            if (pb = strstr(equinox," "))
            {
                *pb++ = NULL;
                iequinox = atoi(equinox);
                strcpy(table,pb);
                if (pb = strstr(table," "))
                {
                    *pb++ = NULL;
                    sprintf(search,"where %s",pb);
                    strtrm(search);
                }
                else
                    search[0] = NULL;
                sprintf(file,"%s%s_par.dat",PRFILES,table);
                strloc(file);
                tfields[0] = NULL;
                if (fp = fopen(file,"r"))
                {
                    while (fgets(buffer,MAXLEN,fp))
                    {
                        if (pb = strstr(buffer,"\n"))
                            *pb = NULL;
                        strtrm(buffer);
                        if (pb = strstr(buffer,"="))
                        {
                            *pb++ = NULL;
                            strtrm(pb);
                            if (!strncasecmp(pb,"SHF",3))
                            {
                                strrpl(buffer,' ','_');
                                strcat(tfields,buffer);
                                strcat(tfields,SEPARATOR);
                            }
                        }
                    }
                    fclose(fp);
                }
                partok(tfields,SEPARATOR,MAXTOK,tfield,&ntfield);
                for (i = 0;i < ntfield;++i)
                {
                    strtrm(tfield[i]);
                    strupc(tfield[i]);
                }
                sprintf(buffer,SQLCOM,iequinox,table,search);
                if (pp = popen(buffer,"r"))
                {
                    count = i = 0;
                    while (iostatus == CSIOS_SUCCESS &&
                    fgets(ibuffer,MAXLEN,pp))
                    {
                        *(strstr(ibuffer,"\n")) = NULL;
                        if (!i && !strcmp(ibuffer,"ADS output"))
                            ++i;
                        else if (i && ++i > 3 && strstr(ibuffer,"|"))
                        {
                            ibuffer[strlen(ibuffer) - 1] = NULL;
                            if (!count)
                            {
                                ++count;
                                if ((iostatus = csios_signal()) ==
                                CSIOS_SUCCESS)
                                {
                                    strcpy(fields,ibuffer);
                                    partok(fields,SEPARATOR,MAXTOK,field,&n);
                                    for (j = 0;j < n;++j)
                                    {
                                        strtrm(field[j]);
                                        if (!strcasecmp(field[j],"RA"))
                                            nra = j;
                                        else if (!strcasecmp(field[j],"DEC"))
                                            ndec = j;
                                        else if (!strcasecmp(field[j],
                                        "POINTRA"))
                                            npra = j;
                                        else if (!strcasecmp(field[j],
                                        "POINTDEC"))
                                            npdec = j;
                                        else if (!strcasecmp(field[j],
                                        "POINT_RA"))
                                            npra = j;
                                        else if (!strcasecmp(field[j],
                                        "POINT_DEC"))
                                            npdec = j;
                                        else if (!strcasecmp(field[j],"CLASS"))
                                            nclass = j;
                                    }
                                }
                            }
                            else
                            {
                                partok(ibuffer,SEPARATOR,MAXTOK,value,&n);
                                for (j = 0;j < n;++j)
                                {
                                    strtrm(value[j]);
                                    if (j == nra)
                                        ra = atof(value[j]);
                                    else if (j == ndec)
                                        dec = atof(value[j]);
                                    else if (j == npra)
                                        pra = atof(value[j]);
                                    else if (j == npdec)
                                        pdec = atof(value[j]);
                                    else if (j == nclass)
                                        iclass = atoi(value[j]);
                                }
                                if (iequinox != 1950)
                                {
                                    if (nra != -1 && ndec != -1)
                                    {
                                        ra = DEG2RAD(ra);
                                        dec = DEG2RAD(dec);
                                        slaPreces("FK5",1950.0,
                                        (double) iequinox,&ra,&dec);
                                        ra = RAD2DEG(ra);
                                        dec = RAD2DEG(dec);
                                    }
                                    if (npra != -1 && npdec != -1)
                                    {
                                        pra = DEG2RAD(pra);
                                        pdec = DEG2RAD(pdec);
                                        slaPreces("FK5",1950.0,
                                        (double) iequinox,&pra,&pdec);
                                        pra = RAD2DEG(pra);
                                        pdec = RAD2DEG(pdec);
                                    }
                                }
                                strcpy(obuffer,LINE);
                                iostatus = csios_output(obuffer,
                                strlen(obuffer) + 1);
                                for (j = 0;iostatus == CSIOS_SUCCESS && j < n;
                                ++j)
                                {
                                    if (j == nra)
                                    {
                                        ra_deg2hms(ra,&hh,&mm,&ss);
                                        sprintf(obuffer,
                                        "%s(%d)=%.7f (%02d %02d %04.1f)",
                                        field[j],iequinox,ra,hh,mm,ss);
                                    }
                                    else if (j == ndec)
                                    {
                                        dec_deg2dms(dec,&sign,&dd,&mm,&ss);
                                        sprintf(obuffer,
                                        "%s(%d)=%.7f (%c%02d %02d %04.1f)",
                                        field[j],iequinox,dec,sign,dd,mm,ss);
                                    }
                                    else if (j == npra)
                                    {
                                        ra_deg2hms(pra,&hh,&mm,&ss);
                                        sprintf(obuffer,
                                        "%s(%d)=%.7f (%02d %02d %04.1f)",
                                        field[j],iequinox,pra,hh,mm,ss);
                                    }
                                    else if (j == npdec)
                                    {
                                        dec_deg2dms(pdec,&sign,&dd,&mm,&ss);
                                        sprintf(obuffer,
                                        "%s(%d)=%.7f (%c%02d %02d %04.1f)",
                                        field[j],iequinox,pdec,sign,dd,mm,ss);
                                    }
                                    else if (j == nclass)
                                    {
                                        hdbi2class(iclass,buffer);
                                        sprintf(obuffer,"%s=%d (%s)",field[j],
                                        iclass,buffer);
                                    }
                                    else
                                    {
                                        sprintf(obuffer,"%s=%s",field[j],
                                        value[j]);
                                        if (ntfield && strlen(value[j]))
                                        {
                                            for (found = 0,k = 0;
                                            !found && k < ntfield;++k)
                                            {
                                                if (!strcasecmp(field[j],
                                                tfield[k]))
                                                {
                                                    shf = atoi(value[j]);
                                                    hdbshf2atim(shf,buffer);
                                                    strcat(obuffer," (");
                                                    strcat(obuffer,buffer);
                                                    strcat(obuffer,")");
                                                    found = 1;
                                                }
                                            }
                                        }
                                    }
                                    iostatus = csios_output(obuffer,
                                    strlen(obuffer) + 1);
                                }
                            }
                        }
                    }
                    if (!count)
                    {
                        strcpy(obuffer,ibuffer);
                        csios_error(obuffer);
                    }
                    pclose(pp);
                }
                else
                {
                    strcpy(obuffer,"error invoking hdbsql");
                    csios_error(obuffer);
                }
            }
        }
        csios_exit();
    }
}
